﻿using System;
using System.Collections;
using System.Collections.Generic;
using UnityEngine;

namespace McDowells
{
    public class Building
    {
        public struct BuildingPiece
        {
            public string FileName;
            public Vector2 Offset;
            public bool Frozen;
            public bool Collisions;
            public int LayerPos;
            public float Scale;
            public int Layer;
            public int LayerSub;
            public int Group;
            public string OriginalItem;
            public string PhysicalProperty;

        }

        public string Name { get; set; }
        public string ModName { get; set; }
        public string PhysicalProperty { get; set; } = "metal";
        public string SpriteImg { get; set; }
        public string OriginalItem { get; set; } = "Metal Cube";
        public string IconPath { get; set; }

        public BuildingPiece MainBuilding = new BuildingPiece();



        public string Description { get; set; } = "";

        private static int BuildingNumber = 0;

        public List<BuildingPiece> Pieces = new List<BuildingPiece>();

        public Building(string identifier, float scale = 1f, string physProp = "Wood", string originalItem = "Stick", int sortingLayer = 0, bool collisions = false, bool frozen = true)
        {
            MainBuilding = new BuildingPiece()
            {
                FileName         = identifier + ".png",
                Offset           = new Vector2(0, 0),
                Collisions       = collisions,
                Frozen           = frozen,
                LayerPos         = 0,
                Scale            = scale,
                PhysicalProperty = physProp,
                OriginalItem     = originalItem,
                Layer            = sortingLayer,
                LayerSub         = 0,

            };
            MainBuilding.FileName = identifier + ".png";


            ModName = "McDowell_Building_" + ++BuildingNumber;

            IconPath = "buildings/" + identifier + ".png";
            Description = "McDowell " + identifier + " " + BuildingNumber;
            Name = identifier.Replace('-', ' ');

            switch (PhysicalProperty)
            {
                case "metal":
                    OriginalItem = "Metal Cube";
                    break;

                case "wood":
                    OriginalItem = "Stick";
                    break;

                default:
                    OriginalItem = "Stick";
                    break;
            }

            Description = Description.Replace('-', ' ');
        }

        public void AddPiece(string fileName, float x = 0, float y = 0, float scale = 1f, bool frozen = false, bool collisions = false, int layerpos = 0, int layer = 0)
        {
            BuildingPiece piece = new BuildingPiece()
            {
                FileName         = fileName,
                Collisions       = collisions,
                Scale            = scale,
                Frozen           = frozen,
                LayerPos         = layerpos,
                Layer            = layer,
                LayerSub         = layerpos,
                Offset           = new Vector2(x, y),
                OriginalItem     = "Stick", //MainBuilding.OriginalItem,
                PhysicalProperty = MainBuilding.PhysicalProperty,
            };

            AddPiece(piece);
        }

        public void AddPiece(BuildingPiece piece)
        {
            Pieces.Add(piece);
        }

        public void Spawn()
        {
            string CatImg = "/buildings/" + MainBuilding.FileName.Replace(".png", "_thumb.png");

            GameObject G2;
            LayerSerialisationBehaviour LSB, LSB2;
            SpriteRenderer SR, SR2;
            PhysicalBehaviour PB, PB2;
            Rigidbody2D R, R2;
            Sprite S;
            EdgeCollider2D E, E2;
            McDBuildingBehaviour McDs;


            ModAPI.Register(
            new Modification()
            {
                OriginalItem        = ModAPI.FindSpawnable("Metal Cube"),
                NameOverride        = "McDonalds1",
                DescriptionOverride = "McDonalds Restaurants",
                CategoryOverride    = ModAPI.FindCategory("McDowells"),
                ThumbnailOverride   = ModAPI.LoadSprite("buildings/mcd_1_thumb.png"),
                AfterSpawn          = (G) =>
                {
                    PB                                = G.GetComponent<PhysicalBehaviour>();
                    R                                 = G.GetComponent<Rigidbody2D>();
                    PB.Properties                     = ModAPI.FindPhysicalProperties("Wood");
                    PB.Disintegratable                = false;
                    PB.Properties.Brittleness         = 0f;
                    PB.Properties.HitVolumeMultiplier = 1f;
                    PB.Properties.Softness            = float.MaxValue;
                    PB.Properties.Buoyancy            = 0.0f;
                    PB.Properties.Conducting          = false;
                    PB.Properties.Flammability        = 1.1f;
                    PB.AbsorbsLasers                  = true;
                    PB.DisplayBloodDecals             = true;
                    PB.InitialMass                    = 100.0f;

                    PB.Properties.BulletSpeedAbsorptionPower = float.MaxValue;

                    R.mass = PB.InitialMass;
                    R.bodyType = RigidbodyType2D.Kinematic;

                    G.GetComponent<SpriteRenderer>().sprite = ModAPI.LoadSprite("buildings/mcd_1.png", 1.5f, false);
                    G.GetComponent<SpriteRenderer>().sortingLayerName = "Top";

                    G.AddComponent<BoxCollider2D>();
                    McDs = G.AddComponent<McDBuildingBehaviour>();
                    McDs.doFreeze = true;

                    G.FixColliders();

                    LSB       = G.AddComponent<LayerSerialisationBehaviour>();
                    LSB.Layer = 10;


                    // ---- Inside 

                    G2 = new GameObject(Name + "0on");
                    G2.transform.SetParent(G.transform);
                    G2.transform.localPosition = new Vector3(-0.13f, -0.386f);

                    SR2                  = G2.AddComponent<SpriteRenderer>();
                    SR2.sprite           = ModAPI.LoadSprite("buildings/" + "mcd_1_inside" + ".png", 1.5f, false);
                    SR2.sortingLayerName = "Bottom";

                    //G2.transform.localScale = new Vector3(1f, 1f);
                    SR2.sortingOrder = 3;


                    G2 = new GameObject(Name + "0xoff");
                    G2.transform.SetParent(G.transform);
                    G2.transform.localPosition = new Vector3(-0.13f, -0.386f);

                    SR2                  = G2.AddComponent<SpriteRenderer>();
                    SR2.sprite           = ModAPI.LoadSprite("buildings/" + "mcd_1_inside_dark" + ".png", 1.5f, false);
                    SR2.sortingLayerName = "Bottom";

                    G2.transform.localScale = new Vector3(1f, 1f);
                    SR2.sortingOrder        = 4;

                    G2 = new GameObject(Name + "1xoff");
                    G2.transform.SetParent(G.transform);
                    G2.transform.localPosition = new Vector3(-0.0f, 2.78f);

                    SR2                  = G2.GetOrAddComponent<SpriteRenderer>();
                    SR2.sprite           = ModAPI.LoadSprite("buildings/" + "mcd_1_roof_dark" + ".png", 1.5f, false);
                    SR2.sortingLayerName = "Bottom";

                    G2.transform.localScale = new Vector3(1f, 1f);
                    SR2.sortingOrder        = 4;

                    


                    //// ---- Roof 

                    S  = ModAPI.LoadSprite("buildings/" + "mcd_1_roof" + ".png", 1.5f, false);
                    G2 = ModAPI.CreatePhysicalObject(Name + "1on", S);

                    G2.transform.SetParent(G.transform);
                    G2.transform.localPosition = new Vector3(-0.0f, 2.78f);

                    PB2 = G2.GetOrAddComponent<PhysicalBehaviour>();
                    //G2.GetOrAddComponent<Rigidbody2D>().bodyType = RigidbodyType2D.Kinematic;
                    SR2 = G2.GetOrAddComponent<SpriteRenderer>();
                    SR2.sortingLayerName = "Bottom";
                    SR2.sortingOrder = 3;

                    G2.GetOrAddComponent<FreezeBehaviour>();

                    foreach (Collider2D c2d in G2.GetComponents<Collider2D>())
                    {
                        c2d.enabled = false;
                    }

                    E2 = G2.AddComponent<EdgeCollider2D>();

                    E2.points = new Vector2[]
                    {
                        new Vector2(-6.0f,-0.1f),
                        new Vector2(0.0f,0.7f),
                        new Vector2(6.2f,0.0f),
                    };

                    E = G2.AddComponent<EdgeCollider2D>();
                    E.points = new Vector2[]
                    {
                        new Vector2(-7.0f,-5.0f),
                        new Vector2(-5.2f,-4.8f),
                        new Vector2(5.2f,-4.8f),
                        new Vector2(7.2f,-5.0f),
                    };

                    // -- Lighting
                    Texture2D tex   = new Texture2D(0, 0);
                    Sprite mySprite = Sprite.Create(tex, new Rect(0.0f, 0.0f, tex.width, tex.height), new Vector2(0.1f, 0.1f), 0.1f);


                    GameObject gameObject = new GameObject("McDFlood", typeof(SpriteRenderer), typeof(AudioSourceTimeScaleBehaviour))
                    {
                        layer = LayerMask.NameToLayer("Objects")
                    };
                    gameObject.GetComponent<SpriteRenderer>().sprite = mySprite;

                    gameObject.AddComponent<Rigidbody2D>();
                    PhysicalBehaviour physicalBehaviour   = gameObject.AddComponent<PhysicalBehaviour>();
                    physicalBehaviour.Properties          = ModAPI.FindPhysicalProperties("Wood");
                    physicalBehaviour.SpawnSpawnParticles = false;
                 
                    McDs.init = true;
                    ModAPI.Notify("<color=yellow>McDonalds</color> is <color=red>CLOSED</color>");
                }
            });

        }
    }


    public class McDBuildingBehaviour : MonoBehaviour
    {
        public bool Power { get; set; } = false;
        public bool Activated;
        public bool PowerOn  = false;
        public bool doFreeze = false;
        public bool doLights = false;
        public bool init     = false;
        private Vector3 OGPos;

        public McLight Lighting;




        public void Update()
        {
            if (init) Initialize();
            
            
        }


        public void Initialize()
        {

            Lighting = gameObject.AddComponent<McLight>();


            init = false;

            Color colorOff1 = new Color(0.5f, 0.5f, 0.7f);
            Color colorOff2 = new Color(1f, 1f, 0.5f);
            Color colorOff3 = new Color(1f, 1f, 1f);
            Color colorOn1  = new Color(1f, 1f, 1f,0.4f);
            Color colorOn2  = new Color(0.5f, 0.5f, 0.4f,0.5f);

            Lighting.AddSimpleLight(gameObject, new Vector3(-6.23f, 1.01f), 5f, 1.5f, colorOff1,1);
            Lighting.AddSimpleLight(gameObject, new Vector3(4.24f, 1.14f), 2.5f, 0.6f, colorOff2,1);
            Lighting.AddSimpleLight(gameObject, new Vector3(-3.57f, 1.08f), 2f, 1.5f, colorOff1,1);
            Lighting.AddSimpleLight(gameObject, new Vector3(-0.48f, -1.16f), 10f, 0.5f, colorOff3,1);
            
            Lighting.AddSimpleLight(gameObject, new Vector3(-1.35f, -0.38f), 10f, 1.5f, colorOn1, 2);
            Lighting.AddSimpleLight(gameObject, new Vector3(4.24f, 1.14f), 2.5f, 1.6f, colorOff2, 2);
            Lighting.AddSimpleLight(gameObject, new Vector3(-0.48f, 1.66f), 2.5f, 3.6f, colorOn1, 2);
            Lighting.AddSimpleLight(gameObject, new Vector3(-6.23f, 1.01f), 5f, 2.5f, colorOn1, 2);

            //McLight.SimpleLight light = Lighting.AddSimpleLight(gameObject, new Vector3(-0.48f, -1.16f), 10f, 5.5f, colorOn1,3);
            //light.Reposition = true;
            //light.Width = 12.5f;

            Lighting.AddSpotLight(gameObject, new Vector3(-4.94f, -2.13f), 0f, 0.1f, colorOn1, 2);
            Lighting.AddSpotLight(gameObject, new Vector3(-2.03f, -2.13f), 0f, 0.2f, colorOn1, 2);
            Lighting.AddSpotLight(gameObject, new Vector3(0.84f, -2.13f), 0f, 0.3f, colorOn1, 2);
            Lighting.AddSpotLight(gameObject, new Vector3(4.46f, -2.13f), 0f, 0.3f, colorOn1, 2);

            McLight.SpotLight spotlight = Lighting.AddSpotLight(gameObject, new Vector3(2.32f, 1.92f), 180f, 0.1f, colorOn2, 2);
            spotlight.Width             = 16f;
            spotlight.Height            = 3f;

            spotlight        = Lighting.AddSpotLight(gameObject, new Vector3(-0.58f, 1.92f), 180f, 0.1f, colorOn2, 2);
            spotlight.Width  = 16f;
            spotlight.Height = 3f;

            spotlight        = Lighting.AddSpotLight(gameObject, new Vector3(-3.36f, 1.92f), 180f, 0.1f, colorOn2, 2);
            spotlight.Width  = 16f;
            spotlight.Height = 3f;


            //spotlight = Lighting.AddSpotLight(gameObject, new Vector3(-0.48f, -1.16f), 180f, 0.1f, colorOn2, 2);
            //spotlight.Reposition        = true;
            //spotlight.Width = 15f;
            //spotlight.Height = 3f;

            if (doFreeze)
            {
                Rigidbody2D[] Arbys = gameObject.transform.GetComponentsInChildren<Rigidbody2D>();

                for (int i = Arbys.Length; --i >= 0;)
                {
                    Arbys[i].bodyType = RigidbodyType2D.Kinematic;
                }
            }

        }


        private void Use()
        {
            SetActivation();



        }

        private void SetActivation()
        {
            Power = !Power;

            Lighting.TurnOnGroup(Power ? 2:1);
            

            foreach (Transform tr in transform.GetComponentInChildren<Transform>())
            {
                if (tr.gameObject.name.Contains("xoff")) tr.gameObject.SetActive(!Power);
            }

            if (Power) ModAPI.Notify("<color=yellow>McDonalds</color> is now <color=green>OPEN</color>");
            else ModAPI.Notify("<color=yellow>McDonalds</color> is <color=red>CLOSED</color>");


        }

        void OnBecameInvisible()
        {
            if (Power)
            {
               if (UnityEngine.Random.Range(1,3) == 1) SpawnRonald();
               else SpawnHamburgler();

            }
        }

        public void SpawnRonald()
        {
            SpawnableAsset ronald   = ModAPI.FindSpawnable("Ronald McDonald");
            
            GameObject instance     = UnityEngine.Object.Instantiate<GameObject>(ronald.Prefab, gameObject.transform.position, Quaternion.identity);
            instance.name = "Ronald McDonald";
            CatalogBehaviour.PerformMod(ronald, instance);

            PersonBehaviour pb = instance.GetComponent<PersonBehaviour>();

            int i = UnityEngine.Random.Range(1, 3);

            if (i == 1) pb.OverridePoseIndex = (int)PoseState.Stumbling;
            else if (i == 2) pb.OverridePoseIndex = (int)PoseState.Walking;
        }

        public void SpawnHamburgler()
        {
            SpawnableAsset ronald = ModAPI.FindSpawnable("The Hamburgler");

            GameObject instance = UnityEngine.Object.Instantiate<GameObject>(ronald.Prefab, gameObject.transform.position, Quaternion.identity);
            instance.name = "The Hamburgler";
            CatalogBehaviour.PerformMod(ronald, instance);

            PersonBehaviour pb = instance.GetComponent<PersonBehaviour>();

            int i = UnityEngine.Random.Range(1,3);

            if (i      == 1) pb.OverridePoseIndex = (int)PoseState.Stumbling;
            else if (i == 2) pb.OverridePoseIndex = (int)PoseState.Walking;

            
        }

    }
}