﻿using System;
using System.Collections;
using System.Collections.Generic;
using UnityEngine;

namespace McDowells
{
    public class McLight : MonoBehaviour
    {

        public class SimpleLight
        {
            public Vector3 Offset   = Vector2.zero;
            public Color Color      = Color.white;
            public float Radius     = 1f;
            public float Brightness = 1f;
            public float Width      = 0f;
            public float Height     = 0f;
            public int Group        = 1;
            public bool AutoFlood   = true;
            public bool Reposition  = false;
            public Transform transform;
            public GameObject Parent;
            public LightSprite Light;
        }

        public class SpotLight
        {
            public Vector3 Offset   = Vector2.zero;
            public Color Color      = Color.white;
            public float Intensity  = 1f;
            public float Rotation   = 0f;
            public float Width      = 0f;
            public float Height     = 0f;
            public int Group        = 1;
            public bool AutoFlood   = true;
            public bool Reposition  = false;
            public Transform transform;
            public GameObject Parent;
            public SingleFloodlightBehaviour Light;

        }

        public bool InitializeLights = false;

        public List<SimpleLight> SimpleLights = new List<SimpleLight>();
        public List<SpotLight> SpotLights     = new List<SpotLight>();

        public int CurrentGroup = 1;

        public float xx, yy;

        public bool xFloodlights = false;
        public MapConfig mapConfig;

        public void Awake()
        {
            foreach (MapConfig _mapConfig in UnityEngine.Object.FindObjectsOfType<MapConfig>())
            {
                mapConfig    = _mapConfig;
                xFloodlights = mapConfig.Settings.Floodlights;
            }

        }

        public void Update()
        {
            if (!InitializeLights) Initialize();

            if (xx != McDowells.gx || yy != McDowells.gy)
            {
                xx = McDowells.gx;
                yy = McDowells.gy;

                foreach (SimpleLight light in SimpleLights)
                {
                    if (light.Reposition)
                    {
                        light.transform.position      = light.Parent.transform.position;
                        light.transform.localPosition = new Vector3(xx, yy);
                    }
                }

                foreach (SpotLight light in SpotLights)
                {
                    if (light.Reposition)
                    {
                        light.transform.position      = light.Parent.transform.position;
                        light.transform.localPosition = new Vector3(xx, yy);
                    }
                }
            }
        }

        public void CheckFloods()
        {
            if (xFloodlights != mapConfig.Settings.Floodlights)
            {
                xFloodlights = mapConfig.Settings.Floodlights;
                if (!xFloodlights) TurnOnGroup(CurrentGroup);
                else TurnOnGroup(0);
            }
        }


        public SimpleLight AddSimpleLight(GameObject parent, Vector3 offset, float radius, float brightness, Color color, int group=1)
        {

            SimpleLight light = new SimpleLight()
            {
                Parent     = parent,
                Offset     = offset,
                Color      = color,
                Radius     = radius,  
                Brightness = brightness,
                Group      = group,
            };

            SimpleLights.Add(light);

            return light;
        }


        public SpotLight AddSpotLight(GameObject parent, Vector3 offset, float rotation, float intensity, Color color, int group = 1)
        {

            SpotLight light = new SpotLight()
            {
                Parent     = parent,
                Offset     = offset,
                Rotation   = rotation,
                Intensity  = intensity,
                Color      = color,
                Group      = group,
            };

            SpotLights.Add(light);

            return light;
        }

        public void Initialize()
        {
            InitializeLights = true;

            foreach (SimpleLight light in SimpleLights)
            {
                SetSimpleLight(light);
            }

            foreach (SpotLight light in SpotLights)
            {
                SetSpotLight(light);
            }

            InvokeRepeating("CheckFloods", 3f, 3f);

        }


        public SpotLight SetSpotLight(SpotLight spotLight)
        {

            SpawnableAsset catLight = ModAPI.FindSpawnable("Flashlight");

            //Vector3 pos            = gameObject.transform.position + spotLight.Offset;
            GameObject instance    = UnityEngine.Object.Instantiate<GameObject>(catLight.Prefab, spotLight.Parent.transform.position, Quaternion.identity);
            PhysicalBehaviour pb   = instance.GetComponent<PhysicalBehaviour>();
            pb.SpawnSpawnParticles = false;

            instance.transform.SetParent(gameObject.transform, false);
            instance.transform.localPosition = spotLight.Offset;
            instance.transform.localRotation = Quaternion.Euler(0,0, spotLight.Rotation);
            if (spotLight.Width > 0f || spotLight.Height > 0f) instance.transform.localScale    = new Vector3(spotLight.Width, spotLight.Height);

            if (instance.TryGetComponent<SpriteRenderer>(out SpriteRenderer sr))
            {
                Texture2D tex   = new Texture2D(0, 0);
                Sprite mySprite = Sprite.Create(tex, new Rect(0.0f, 0.0f, tex.width, tex.height), new Vector2(0.1f, 0.1f), 0.1f);
                sr.sprite       = mySprite;
            }
            
            if (instance.TryGetComponent<SingleFloodlightBehaviour>(out SingleFloodlightBehaviour light))
            {
                spotLight.Light = light;

                light.Activated = spotLight.Group == CurrentGroup;

                if (xFloodlights) light.Activated = false;

                //light.enabled = false;

                foreach (SingleFloodlightBehaviour.Highlight highlight in light.Highlights)
                {
                    highlight.Colour    = spotLight.Color;
                    highlight.Intensity = spotLight.Intensity;
                }
            }

            spotLight.transform = instance.transform;
            Rigidbody2D rb      = instance.GetComponent<Rigidbody2D>();
            rb.bodyType         = RigidbodyType2D.Static;

            foreach (Collider2D c2d in instance.GetComponents<Collider2D>())
            {
                c2d.enabled = false;
            }

            return spotLight;
        }


        public SimpleLight SetSimpleLight(SimpleLight simpleLight)
        {
            LightSprite light = UnityEngine.Object.Instantiate(
                Resources.Load<GameObject>("Prefabs/ModLightPrefab"), gameObject.transform
            ).GetComponent<LightSprite>();

            light.transform.SetParent(gameObject.transform,false);
            
            light.transform.localPosition = simpleLight.Offset;

            if (simpleLight.Width > 0f || simpleLight.Height > 0f) light.transform.localScale = new Vector3(simpleLight.Width, simpleLight.Height);

            light.Color                   = simpleLight.Color;
            light.Radius                  = simpleLight.Radius;
            light.Brightness              = simpleLight.Brightness;
            
            simpleLight.transform = light.transform;
            simpleLight.Light     = light;

            simpleLight.Light.SpriteRenderer.forceRenderingOff = (simpleLight.Group != CurrentGroup);

            if (xFloodlights) simpleLight.Light.SpriteRenderer.forceRenderingOff = true;


            return simpleLight;
        
        }

        public void TurnOnSpotLights()
        {
            foreach (SpotLight spotLight in SpotLights)
            {
                if (spotLight.Group == CurrentGroup)
                {
                    spotLight.Light.Activated = true;
                    if (xFloodlights)
                    {
                        spotLight.Light.Activated = false;

                    }
                }
                else
                {
                        spotLight.Light.Activated = false;
                }
            }
        }

        public void TurnOnGroup(int groupId, bool turnOffOthers = true)
        {
            CurrentGroup = groupId;

            foreach (SimpleLight simpleLight in SimpleLights)
            {
                if (simpleLight.Group == groupId)
                {
                    simpleLight.Light.SpriteRenderer.forceRenderingOff = false;
                    if (xFloodlights) simpleLight.Light.SpriteRenderer.forceRenderingOff = true;
                } else
                {
                    if (turnOffOthers || xFloodlights) simpleLight.Light.SpriteRenderer.forceRenderingOff = true;
                }
            }

            Invoke("TurnOnSpotLights", 0.1f);

            
        }

        
    }

}